#include "Time.h"
#include "Bus.h"
#include "Pin.h"
#include "CAN.h"
#include "DIORaw.h"
#include "DIO.h"
#include "TWI.h"
#include "CacheBus.h"
#include "tsDIO24DIORaw.c"
#include "DummyDIO.c"
#include "PhysicalDIO.h"
#include "arch.h"

static tsDIO24DIORaw dio24[4];
static PhysicalDIO pdio24[4];
static unsigned char DIO24DIOCapabilities[24];
static CacheBus tsDIO24DIOCacheBus;
static DummyDIO r8dummyDIO[4];

unsigned tsDIO24DIOCacheBus_WO[4];
unsigned tsDIO24DIOCacheBus_IBit[4];
unsigned tsDIO24DIOCacheBus_OBit[4];
unsigned tsDIO24DIOCacheBus_icache[4];
unsigned tsDIO24DIOCacheBus_ocache[4];
unsigned tsDIO24DIOCacheBus_ocache0[4];

System *tsDIO24_SystemInit(System *s,int inst) {
  return s;
}

Time *tsDIO24_TimeInit(Time *t,int inst) {
  return t;
}

Bus *tsDIO24_BusInit(Bus *bus,int inst) {
  if (inst == 0) {
    tsDIO24DIOCacheBus_IBit[1] = 0xFF;
    tsDIO24DIOCacheBus_IBit[2] = 0xFF;
    tsDIO24DIOCacheBus_IBit[3] = 0xFF;
    tsDIO24DIOCacheBus_OBit[0] = 3;
    tsDIO24DIOCacheBus_OBit[1] = 0xFF;
    tsDIO24DIOCacheBus_OBit[2] = 0xFF;
    tsDIO24DIOCacheBus_OBit[3] = 0xFF;
    tsDIO24DIOCacheBus.Length = 4;
    tsDIO24DIOCacheBus.Width = 8;
    tsDIO24DIOCacheBus.Offset = 4;
    tsDIO24DIOCacheBus.WO = tsDIO24DIOCacheBus_WO;
    tsDIO24DIOCacheBus.IBit = tsDIO24DIOCacheBus_IBit;
    tsDIO24DIOCacheBus.OBit = tsDIO24DIOCacheBus_OBit;
    tsDIO24DIOCacheBus.icache = tsDIO24DIOCacheBus_icache;
    tsDIO24DIOCacheBus.ocache = tsDIO24DIOCacheBus_ocache;
    tsDIO24DIOCacheBus.ocache0 = tsDIO24DIOCacheBus_ocache0;
    return CacheBusInit(&tsDIO24DIOCacheBus,BusInit(2));
  }
  return bus;
}

Pin *tsDIO24_PinInit(Pin *pin,int inst) {
  return pin;
}

CAN *tsDIO24_CANInit(CAN *can,int inst) {
  return can;
}

DIORaw *tsDIO24_DIORawInit(DIORaw *dioraw0,int inst) {
  Bus *muxbus = BusInit(2);
  int i=0;
  DIORaw *dioraw;

  if (inst >= 0 && inst <= 3) {
    muxbus->API->Lock(muxbus,0,0);
    muxbus->API->BitClear16(muxbus,2,9); // make sure ISA reset de-asserted
    muxbus->API->Unlock(muxbus,0,0);
    dioraw = tsDIO24DIORawInit(&dio24[i],tsDIO24_BusInit(0,0),0x100+inst*8);
    if (dioraw) {
      return dioraw;
    } else {
      return 0;
    }
  }
  return dioraw0;
}

DIO *tsDIO24_DIOInit(DIO *dio,int inst) {
  DIORaw *dioraw;
  int i;
  if (inst >= 0 && inst <= 3) {
    dioraw = tsDIO24_DIORawInit(0,inst);
    if (dioraw) {
      for (i=0;i<4;i++) {
	DIO24DIOCapabilities[i] = DIO_OUTPUT_ONLY;
      }
      for (i=4;i<8;i++) {
	DIO24DIOCapabilities[i] = DIO_CAN_DRIVE_LOW;
      }
      for (i=8;i<24;i++) {
	DIO24DIOCapabilities[i] = DIO_NORMAL;
      }
      pdio24[inst].Capabilities = DIO24DIOCapabilities;
      //return PhysicalDIOInit(&pdio24[inst],BusInit(2),dioraw);
      return PhysicalDIOInit(&pdio24[inst],tsDIO24_BusInit(0,0),dioraw);
    } else {
      return DummyDIOInit(&r8dummyDIO,8);
    }
  }
  return dio;
}

TWI *tsDIO24_TWIInit(TWI *twi,int inst) {
  return twi;
}

SPI *tsDIO24_SPIInit(SPI *spi,int inst) {
  return spi;
}

AIO *tsDIO24_AIOInit(AIO *aio,int inst) {
  return aio;
}
#include <stdio.h>
void tsDIO24_ArchInit() {
  if (!DIOInit(0)) {
    fprintf(stderr,"This binary was compiled with TS-DIO24 support\n");
    fprintf(stderr,"Error: No TS-DIO24 detected\n");
  }
}

ArchInfo tsdio24_ArchInfo = {
  .Instances={0,0,0,0,1,1,0,0,0},
  .CANBusNum=0,
  .ArchInit=tsDIO24_ArchInit,
  .SystemInit=tsDIO24_SystemInit,
  .TimeInit=tsDIO24_TimeInit,
  .BusInit=tsDIO24_BusInit,
  .PinInit=tsDIO24_PinInit,
  .CANInit=tsDIO24_CANInit,
  .DIORawInit=tsDIO24_DIORawInit,
  .DIOInit=tsDIO24_DIOInit,
  .SPIInit=tsDIO24_SPIInit,
  .TWIInit=tsDIO24_TWIInit,
  .AIOInit=tsDIO24_AIOInit
  //.EDIOInit=tsDIO24_EDIOInit
};

// Author: Michael Schmidt (michael@embeddedARM.com)
// Copyright (c) 2012, Technologic Systems, All Rights Reserved
// Refer to the COPYRIGHT file provided with this project for licensing terms.
